// Copyright (c) 2022 Vinícius dos Santos Oliveira
// SPDX-License-Identifier: MIT OR BSL-1.0

#pragma once

#include <emilua/core.hpp>

#if BOOST_OS_WINDOWS
#include <boost/nowide/convert.hpp>
#endif

namespace emilua {

#if BOOST_OS_WINDOWS
template<class... Args>
auto widen_on_windows(Args&&... args)
    -> decltype(nowide::widen(std::forward<Args>(args)...))
{
    return nowide::widen(std::forward<Args>(args)...);
}

template<class... Args>
auto narrow_on_windows(Args&&... args)
    -> decltype(nowide::narrow(std::forward<Args>(args)...))
{
    return nowide::narrow(std::forward<Args>(args)...);
}
#else
template<class T>
auto widen_on_windows(T&& t) -> decltype(std::forward<T>(t))
{
    return std::forward<T>(t);
}

template<class... Args>
std::string narrow_on_windows(Args&&... args)
{
    return std::string(std::forward<Args>(args)...);
}
#endif

} // namespace emilua
